require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
  class Game
    attr_reader :title
    attr_reader :players
    def initialize(title)
      @title = title
      @players =[]
    end

    def load_players(from_file)
      puts "Loading players ..........."
      CSV.foreach(from_file) do |row|
        add_player(Player.new(row[0],row[1].to_i))
      end
    end

    def high_score_entry(player)
      "#{player.name.ljust(20,'.')} #{player.score}"
    end

    def save_high_scores(save_file="high_scores.txt")
      File.open(save_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        sorted_players = @players.sort {|a,b| b.score <=> a.score}
        sorted_players.each do |p|
          file.puts high_score_entry(p)
        end
      end
    end

    def total_points
      @players.reduce(0) { |sum, p|  sum + p.points }
    end

    def print_name_and_health(p)
      puts "#{p.name} (#{p.health})"
    end

    def add_player(player)
      @players << player
    end

    def print_stats
      strong_players, wimpy_players = @players.partition{|p| p.strong?}
      puts "\n#{@title} Statistics:"
      puts "\n#{strong_players.length} strong players:"
      strong_players.each do |p|
        print_name_and_health(p)
      end

      puts "\n#{wimpy_players.length} wimpy players:"
      wimpy_players.each do |p|
        print_name_and_health(p)
      end

      puts "\n#{@title} High Scores:"
      sorted_players = @players.sort {|a,b| b.score <=> a.score}
      sorted_players.each do |p|
        puts high_score_entry(p)
      end

      @players.each do |p|
        puts "\n#{p.name}'s point totals:"
        p.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{p.score} grand total points"
      end

      puts "#{total_points} total points from treasures found"

    end

    def play(rounds)
      puts "There are #{@players.size} players in #{@title}: "

      @players.each do |p|
        puts p
      end

      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.length} treasures to be found:"
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end

      1.upto(rounds) do |r|
        if block_given?
          break if yield
        end
        puts "\nRound Number: #{r}"
        @players.each do |p|
          GameTurn.take_turn(p)
          puts p
        end
      end
    end
  end

  if __FILE__ ==$PROGRAM_NAME
    game = Game.new("Kerplunk")
    player1 = Player.new("mouse")
    player2 = Player.new("cat")
    game.add_player(player1)
    game.add_player(player2)
    puts game.players
    game.play(2)
  end
end
