require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame
  class Player
    attr_accessor :health
    attr_accessor :name
    include Playable

    def initialize(name, health=100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end

    def self.from_csv (line)
      name, health = line.split(',')
      Player.new(name, Integer(health))
    end

    def each_found_treasure
      @found_treasures.each do |name, points|
        treasure = Treasure.new(name, points)
        yield(treasure)
      end
    end

    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{@name} found a #{treasure.name} worth #{treasure.points} points"
  #    puts "\n#{@name}'s treasures: #{@found_treasures}'"
    end

    def points
      @found_treasures.values.reduce(0, :+)
    end

    def to_s
      "I'm #{@name} with a health = #{@health}, points = #{points}, and score = #{score}"
    end

    def score
      @health+points
    end

    def name=(new_name)
      @name = new_name.capitalize
    end
  end

  if __FILE__ == $PROGRAM_NAME
    player = Player.new("moe",100)
    puts player.points
    puts player.name
    puts player.health
    player.w00t
    puts player.health
    player.blam
    puts player.health
  end
end
